# TODO: Replace lambda match functions with machine learning lol
stages = {
    "Battlefield": lambda str: "battle" in str,
    "Dream Land N64": lambda str: "land" in str,
    "Final Destination": lambda str: "final" in str or str == "fd",
    "Fountain of Dreams": lambda str: "fountain" in str or str == "fod",
    "Yoshi's Story": lambda str: "yoshi" in str,
    "Pokemon Stadium": lambda str: "pokemon" in str or "stadium" in str or str == "ps",
    # TODO: Add non-legal stages
}
characters = {
    "Bowser": lambda str: "bowser" in str,
    "Captain Falcon": lambda str: "falcon" in str or str == "cf",
    "Donkey Kong": lambda str: "donkey" in str or str == "dk",
    "Dr. Mario": lambda str: "dr" in str,
    "Falco": lambda str: "falco" in str and "falcon" not in str,
    "Fox": lambda str: "fox" in str,
    "Ganondorf": lambda str: "ganon" in str,
    "Ice Climbers": lambda str: "ic" in str,
    "Jigglypuff": lambda str: "jig" in str or "puff" in str,
    "Kirby": lambda str: "kirby" in str,
    "Link": lambda str: "link" in str and "y" not in str,
    "Luigi": lambda str: "luigi" in str,
    "Mario": lambda str: "mario" in str and "dr" not in str,
    "Marth": lambda str: "marth" in str,
    "Mewtwo": lambda str: "mew" in str,
    "Mr. Game & Watch": lambda str: "game" in str or "&" in str,
    "Ness": lambda str: "ness" in str,
    "Peach": lambda str: "peach" in str or "daisy" in str,
    "Pichu": lambda str: "pichu" in str,
    "Pikachu": lambda str: "pikachu" in str,
    "Roy": lambda str: "roy" in str,
    "Samus": lambda str: "samus" in str,
    "Sheik": lambda str: "sheik" in str or "shiek" in str,
    "Young Link": lambda str: "link" in str and "y" in str,
    "Yoshi": lambda str: "yoshi" in str,
    "Zelda": lambda str: "zelda" in str
    # TODO: Add SSBU characters
}
rounds = {
    "Winners Quarters": lambda str: "winner" in str and "quarter" in str,
    "Winners Semis": lambda str: "winner" in str and "semi" in str,
    "Winners Finals": lambda str: "winner" in str and "final" in str,
    "Losers Eighths": lambda str: "loser" in str and "eight" in str,
    "Losers Quarters": lambda str: "loser" in str and "quarter" in str,
    "Losers Semis": lambda str: "loser" in str and "semi" in str,
    "Losers Finals": lambda str: "loser" in str and "final" in str,
    "Grand Finals": lambda str: "grand" in str and "final" in str,
}
sponsors = {
    "Team Liquid": lambda str: "liquid" in str or "tl" in str,
    "Alliance": lambda str: "[A]" in str or "alliance" in str,
    "Counter Logic Gaming": lambda str: "clg" in str or ("counter" in str and "logic" in str),
    "Cloud 9": lambda str: "C9" in str or "cloud" in str,

}


def character(str):
    str = str.lower()
    for char, compare in characters.items():
        if compare(str):
            return char
    return None


def stage(str):
    str = str.lower()
    for stage, compare in stages.items():
        if compare(str):
            return stage
    return None


def round(str):
    str = str.lower()
    for round, compare in rounds.items():
        if compare(str):
            return round
    return None


import re
        
class Player():
    __slots__ = ['sponsor', 'name', 'characters']

    def __init__(self, sponsor="", name="", characters=[]):
        self.sponsor = sponsor
        self.name = name
        self.characters = characters

class Vod():
    def __init__(self, vod_id=""):
        self.vod_id = vod_id
        self.video_id = ""
        self.platform = ""

        self.date = ""
        self.tournament = ""

        self.player1 = Player()
        self.player2 = Player()

        self.round = ""
        self.format = ""

        self.games = []

    def parse_title(self, vod_title: str):
        title_pattern = "\(([0-9]*-[0-9]*-[0-9]*)\) (.*) vs (.*) \[(.*)\] - (.*) - (.*)"

        m = re.search(title_pattern, vod_title)

        self.date = m.group(1).strip()
        self.tournament = m.group(5).strip()

        self.player1.name = m.group(2).strip()
        self.player2.name = m.group(3).strip()

        self.round = m.group(6).strip()
        self.format = m.group(4).strip()

    def values(self):
        return [self.vod_id, self.video_id, self.platform, self.date,
                self.tournament, self.player1.name, self.player1.characters,
                self.player2.name, self.player2.characters,
                self.round, self.format]

    def formatted_title(self):
        title = ""

        if self.date:
            title += "[" + repr(self.date) + "] "
        if self.tournament:
            title += self.tournament + " - "

        if self.player1.sponsor:
            title += self.player1.sponsor + " | "
        if self.player1.name:
            title += self.player1.name + " "
        else:
            title += "Unknown "
        if self.player1.characters:
            title += "(" + ", ".join(self.player1.characters) + ") "

        title += "vs "

        if self.player2.sponsor:
            title += self.player2.sponsor + " | "
        if self.player2.name:
            title += self.player2.name + " "
        else:
            title += "Unknown "
        if self.player2.characters:
            title += "(" + ", ".join(self.player2.characters) + ") "

        if self.round:
            title += "- " + self.round + " "
        if self.format:
            title += "- " + self.format

        return title.strip()

    def __repr__(self):
        return repr(self.formatted_title())



class Error(Exception):
    pass


class InvalidVideoError(Error):
    def __init__(self, *args):
        message = "Invalid Video: Vods.co vod '%s' links to invalid '%s' video: '%s'" % args
        super(InvalidVideoError, self).__init__(message)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='vodscrepe',
    version='0.0.8',
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    description='https://vods.co/ vod scraper',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Dawson Booth',
    author_email='pypi@dawsonbooth.com',
    url='https://github.com/dawsonbooth/vodscrepe',
    packages=setuptools.find_packages(),
    python_requires='>=3.6',
    license='MIT',
)



import gzip
import re
import urllib.request
from urllib.error import HTTPError, URLError

import lxml
from bs4 import BeautifulSoup, SoupStrainer
from tqdm import tqdm

from .aliases import *
from .errors import *
from .vod import Vod


def urlopen(url, debug=False):
    headers = {'Accept': 'application/xhtml+xml',
               'Accept-Encoding': 'gzip'}
    req = urllib.request.Request(url, headers=headers)

    response = None
    while response is None:
        try:
            response = urllib.request.urlopen(req)
        except (URLError, HTTPError):
            if debug:
                tqdm.write("Connection Error: Reconnecting to '%s'" % url)
            continue
    return gzip.decompress(response.read())


# TODO: Scrape vods.co for list of chars, player names, tournaments, etc.

class Scraper():
    def __init__(self, game, debug=False):
        self.home_url = "https://vods.co"
        self.game = game
        self.page = 0
        self.debug = debug

    def scrape_vod(self, vod_url: str, platform='youtube'):
        vod_content = urlopen(vod_url, self.debug)
        vod_strainer = SoupStrainer('div')
        vod_soup = BeautifulSoup(vod_content, "lxml", parse_only=vod_strainer)
        video_tag = vod_soup.find("div", class_="js-video widescreen", id="g1")
        if video_tag is None:
            return None
        title_tag = vod_soup.find("div", class_="block-title")
        vod_title = title_tag.get_text().replace('  ', ' ')

        # Create vod object
        vod = Vod()

        # VOD properties
        vod.vod_id = vod_url[18:]
        vod.video_id = video_tag["data-vod"].split("?")[0]
        vod.platform = 'youtube'
        # TODO: Support Twitch vods

        # Parse title for svp Vod info
        vod.parse_title(vod_title)

        return vod

    def _scrape_page(self, page_url, platform='youtube'):
        # Page soup
        page_content = urlopen(page_url, self.debug)
        page_strainer = SoupStrainer("tr", class_=re.compile("recency"))
        page_soup = BeautifulSoup(
            page_content, "lxml", parse_only=page_strainer)

        if page_soup.findChild('tr') is None:
            yield None

        for row in page_soup.findChildren("tr"):
            cells = [cell for cell in row.findChildren("td")]

            # Check Match Format
            match_format = cells[3].get_text().strip()
            if re.match("Bo\d", match_format) is None:
                continue

            # Create VOD
            try:
                vod = self.scrape_vod(
                    cells[1].a["href"], platform=platform)
            except InvalidVideoError as e:
                if self.debug:
                    tqdm.write(e)
                continue

            if vod is None:
                continue

            # Characters
            title_tag = cells[1].a.span
            num_chars_p1 = str(title_tag).split("vs")[0].count("img")
            characters = [char_img["src"][24:-4]
                          for char_img in title_tag.find_all(
                "img", recursive=False)]
            vod.player1.characters = [character(
                c) for c in characters[:num_chars_p1]]
            if None in vod.player1.characters:
                vod.player1.characters = []
            vod.player2.characters = [character(
                c) for c in characters[num_chars_p1:]]
            if None in vod.player2.characters:
                vod.player2.characters = []

            yield vod

    def scrape(self, pages=range(300), platform='youtube', event='',
               player1='', player2='', character1='', character2='',
               show_progress=True):

        page_url = self.home_url + "/" + self.game
        if player1:
            page_url += "/player/" + player1
        if player2:
            page_url += "/player2/" + player2
        if event:
            page_url += "/event/" + event
        if character1:
            page_url += "/character/" + character1
        if character2:
            page_url += "/character2/" + character2

        page_url = page_url.replace(' ', '%20')

        if show_progress:
            pages = tqdm(pages, position=1, unit='pages', desc="All vods")

        for page in pages:
            self.page = page
            url = page_url if page == 0 else page_url + "?page=" + str(page)

            vods = self._scrape_page(url, platform=platform)
            if show_progress:
                vods = tqdm(vods, position=0, unit='vods',
                            desc="Page %d" % page, total=60)

            for vod in vods:
                if vod is None:
                    return None
                yield vod
